#===============================================================================
#
# Copyright (c) 2014 Qualcomm Technologies, Inc. All Rights Reserved.
# Qualcomm Technologies Proprietary and Confidential.
#
#===============================================================================

import hashlib
import struct
import defines

from sectools.features.fuse_common.parsegen.secdat.secdatplugin import SecdatPluginIntf_Abs
from sectools.features.fuse_common.parsegen import BaseDataParseGen
from sectools.common.utils.c_logging import logger
from sectools.features.fuse_common.datamodel import auto_gen_model
from sectools.common.utils.c_misc import abstractclassmethod


class SecdatParseGen(BaseDataParseGen, SecdatPluginIntf_Abs):

    def __init__(self, header_info='', header_version=1,
                 fuse_list_header_version=1):
        BaseDataParseGen.__init__(self)
        self._header = None
        self._fuse_header = None
        self._fuse_list = []
        self._footer = None
        self.header_info = header_info
        self.header_version = header_version
        self.fuse_list_header_version = fuse_list_header_version
        self.file_path = ''

    def add_fuse(self, fuse):
        self._fuse_list.append(fuse)
        self._fuse_list.sort(key=lambda x: x.raw_row_address)

    @classmethod
    def get_plugin_id(cls):
        return cls.file_version()

    @classmethod
    def get_plugin_manager(cls):
        return secdatparsegen_mgr

    @classmethod
    def get_region_enum_from_id(cls, region_id):
        return defines.fuseprov_region_etype_dict[defines.secdat_region_etype_dict[region_id]]

    #--------------------------------------------------------------------------
    # Properties & Functions that must be implemented by the derived classes
    #--------------------------------------------------------------------------

    @abstractclassmethod
    def file_version(cls):
        raise RuntimeError('Class ' + cls.__name__ + ' must implement the class method file_type.')

    @abstractclassmethod
    def create_header(self, data_size, info, feature):
        raise RuntimeError('Method must be implemented by the derived class')

    def validate_header(self, header, file_size):
        raise RuntimeError('Method must be implemented by the derived class')

    def parse_secdat_hdr_data(self, file_data):
        raise RuntimeError('Method must be implemented by the derived class')

    def write(self, data_model, input_file_path, output_file_path, common_file_path, exist_file_path, feature):
        raise RuntimeError('Method must be implemented by the derived class')

    def get_operation_enum_from_id(self, operation_id):
        return defines.fuseprov_operation_etype_dict[defines.secdat_operation_etype_dict[operation_id]]

    def get_region_id_from_enum(self, enum_value):
        # Look for the fuseprov id based on the enum
        fuseprov_id = ''
        for key, value in defines.fuseprov_region_etype_dict.items():
            if value == enum_value:
                fuseprov_id = key
                break
        else:
            raise RuntimeError('Unknown enum id: ' + str(enum_value))

        # Look for the secdat id based on the enum
        secdat_id = ''
        for key, value in defines.secdat_region_etype_dict.items():
            if value == fuseprov_id:
                secdat_id = key
                break
        else:
            raise RuntimeError('Unknown fuseprov id: ' + str(fuseprov_id) + ' for enum: ' + str(enum_value))

        # Return the secdat id
        return secdat_id

    def get_operation_id_from_enum(self, enum_value):
        # Look for the fuseprov id based on the enum
        fuseprov_id = ''
        for key, value in defines.fuseprov_operation_etype_dict.items():
            if value == enum_value:
                fuseprov_id = key
                break
        else:
            raise RuntimeError('Unknown enum id: ' + str(enum_value))

        # Look for the secdat id based on the enum
        secdat_id = ''
        for key, value in defines.secdat_operation_etype_dict.items():
            if value == fuseprov_id:
                secdat_id = key
                break
        else:
            raise RuntimeError('Unknown fuseprov id: ' + str(fuseprov_id) + ' for enum: ' + str(enum_value))

        # Return the secdat id
        return secdat_id

    def create_fuse_header(self, fuse_count):
        fuse_header = fuseprov_qfuse_list_hdr_type()
        fuse_header.revision = defines.CURRENT_SUPPORTED_QFUSELIST_REVISION
        fuse_header.fuse_count = fuse_count
        fuse_header.size = fuseprov_qfuse_entry.PACK_SIZE * fuse_count
        return fuse_header

    def create_fuse_entry(self, region_type, address, operation, value):
        fuse_entry = fuseprov_qfuse_entry()
        fuse_entry.region_type = region_type
        fuse_entry.raw_row_address = int(address, 16)
        fuse_entry.operation = operation
        fuse_entry.msb_val = int(value[2:10], 16)
        fuse_entry.lsb_val = int(value[10:18], 16)
        return fuse_entry

    def create_footer(self, contents):
        footer = fuseprov_secdat_footer_type()
        hasher = hashlib.sha256()
        hasher.update(contents)
        footer.hash = hasher.digest()
        return footer

    def validate_fuse_header(self, fuse_header, fuse_data):
        if not (fuse_header.revision == defines.CURRENT_SUPPORTED_QFUSELIST_REVISION and
                fuse_header.size == fuseprov_qfuse_entry.PACK_SIZE * fuse_header.fuse_count and
                fuse_header.size == len(fuse_data)):
            raise RuntimeError('Fuse Header Mismatch')
        pass

    def read(self, file_path):
        fd = open(file_path, 'rb')
        file_data = fd.read()
        self.parse(file_data)
        self.update_data_model()

    def validate_fuse_entry(self, fuse_entry):
        if not (fuse_entry.region_type in defines.fuseprov_region_etype_dict.values() and
                fuse_entry.operation in defines.fuseprov_operation_etype_dict.values()):
            raise RuntimeError('Fuse Entry Mismatch')
        pass

    def validate_footer(self, footer, contents):
        hasher = hashlib.sha256()
        hasher.update(contents)
        if not (footer.hash == hasher.digest()):
            raise RuntimeError('Footer Mismatch')
        pass

    def generate(self, file_path):
        file_data = (self._header.get_packed_data() +
                     self._fuse_header.get_packed_data())
        for fuse in self._fuse_list:
            file_data += fuse.get_packed_data()
        file_data += self._footer.get_packed_data()

        # Generate the file
        fd = open(file_path, 'wb')
        fd.write(file_data)
        fd.close()

        self.file_path = file_path

    def __repr__(self):
        retval = ''
        retval += ('Secdat properties: ' + '\n' + '\n'
                   '' + 'File path: ' + self.file_path + '\n' + '\n'
                   '' + 'Header: ' + '\n'
                   '' + logger.increase_indent(repr(self._header)) + '\n'
                   '' + 'Fuse Header: ' + '\n'
                   '' + logger.increase_indent(repr(self._fuse_header)) + '\n'
                   '' + 'Footer: ' + '\n'
                   '' + logger.increase_indent(repr(self._footer)) + '\n')
        for fuse in self._fuse_list:
            retval += ('' + 'Fuse: ' + '\n'
                       '' + logger.increase_indent(repr(fuse)) + '\n')
        return retval.strip()

    def parse(self, file_data):

        if file_data is None:
            return

        # Get the header data specific to sec.dat revision
        header_data, file_data = self.parse_secdat_hdr_data(file_data)

        # Get the footer data
        footer_data = file_data[len(file_data) - fuseprov_secdat_footer_type.PACK_SIZE:]
        file_data = file_data[:len(file_data) - len(footer_data)]
        self._footer = fuseprov_secdat_footer_type(footer_data)
        self.validate_footer(self._footer, header_data + file_data)

        # Get the fuse header data
        fuse_header_data = file_data[:fuseprov_qfuse_list_hdr_type.PACK_SIZE]
        file_data = file_data[fuseprov_qfuse_list_hdr_type.PACK_SIZE:]
        self._fuse_header = fuseprov_qfuse_list_hdr_type(fuse_header_data)
        self.validate_fuse_header(self._fuse_header, file_data)
        self.fuse_list_header_version = self._fuse_header.revision

        # Get the fuses
        self._fuse_list = []
        for idx in range(0, self._fuse_header.fuse_count):
            fuse_data = file_data[:fuseprov_qfuse_entry.PACK_SIZE]
            file_data = file_data[fuseprov_qfuse_entry.PACK_SIZE:]
            fuse = fuseprov_qfuse_entry(fuse_data)
            self.add_fuse(fuse)
            self.validate_fuse_entry(fuse)


    def update_from_data_model(self, data_model, feature):

        self._fuse_list = []
        for module in data_model.module:
            for fuse_region in module.fuse_region:
                for fuse in fuse_region.fuse:
                    if fuse.ignore:
                        logger.info('Ignoring fuse: ' + str(fuse))
                        continue
                    fuse_entry = self.create_fuse_entry(self.get_region_enum_from_id(fuse_region.id),
                                                        fuse.address,
                                                        self.get_operation_enum_from_id(fuse.operation),
                                                        fuse.value)
                    self.add_fuse(fuse_entry)

        self._fuse_header = self.create_fuse_header(len(self._fuse_list))
        data = self._fuse_header.get_packed_data()
        for fuse in self._fuse_list:
            data += fuse.get_packed_data()

        self._header = self.create_header(len(data) + fuseprov_secdat_footer_type.PACK_SIZE, self.header_info, feature)
        data = self._header.get_packed_data() + data
        self._footer = self.create_footer(data)

    def update_data_model(self):
        PSEUDO_NAME = 'UNKNOWN_NAME'
        PSEUDO_ADDRESS = '0x11223344'
        PSEUDO_LSB_ID = 'LSB'
        PSEUDO_MSB_ID = 'MSB'
        PSEUDO_LSB_BITS = '31:0'
        PSEUDO_MSB_BITS = '63:32'
        PSEUDO_OWNER = auto_gen_model.defines.OWNER_OEM

        # Create a dictionary of fuse regions
        fuse_data = dict()
        normalizer = auto_gen_model.Cfg_Validations.normalize_value
        bit_length_32_bit = auto_gen_model.BIT_LENGTH_32_BITS
        for fuse in self._fuse_list:
            region_type = self.get_region_id_from_enum(fuse.region_type)
            operation = self.get_operation_id_from_enum(fuse.operation)
            address = normalizer(hex(fuse.raw_row_address), bit_length_32_bit)
            value = (normalizer(hex(fuse.msb_val), bit_length_32_bit) +
                     normalizer(hex(fuse.lsb_val), bit_length_32_bit)[2:])
            retval = fuse_data.setdefault(region_type, [])
            retval.append((address, operation, value))

        # Create a data model
        data_model = auto_gen_model.Cfg_Fuseblower()

        # Add the top level module
        security_module = auto_gen_model.Cfg_Module()
        security_module.id = PSEUDO_NAME
        security_module.base_address = PSEUDO_ADDRESS
        data_model.add_module(security_module)

        for region, region_data in fuse_data.items():
            # Create a new fuse region and add to the security module
            fuse_region = auto_gen_model.Cfg_Fuse_Region()
            fuse_region.id = region
            security_module.add_fuse_region(fuse_region)

            for idx, fuse_info in enumerate(region_data):
                # Create new fuses per address
                fuse = auto_gen_model.Cfg_Fuse()
                fuse.n = idx
                fuse.address = fuse_info[0]
                fuse.ignore = False
                fuse.operation = fuse_info[1]
                fuse.value = fuse_info[2]

                # Create the two basic fields for LSB & MSB
                field_LSB = auto_gen_model.Cfg_Field()
                field_LSB.id = PSEUDO_LSB_ID
                field_LSB.bits = PSEUDO_LSB_BITS
                field_LSB.owner = PSEUDO_OWNER

                field_MSB = auto_gen_model.Cfg_Field()
                field_MSB.id = PSEUDO_MSB_ID
                field_MSB.bits = PSEUDO_MSB_BITS
                field_MSB.owner = PSEUDO_OWNER

                fuse.add_field(field_MSB)
                fuse.add_field(field_LSB)

                # Add the fuse to the fuse region
                fuse_region.add_fuse(fuse)

        self.data_model = data_model


class fuseprov_qfuse_list_hdr_type(object):

    # Structure object to align and package the fuse list header
    PACK_FMT = struct.Struct('III' +
                             'I' * defines.FUSEPROV_RESERVED_MAX_SIZE)

    # Size of the header
    PACK_SIZE = ((4 * 3) +
                (4 * defines.FUSEPROV_RESERVED_MAX_SIZE))

    def __init__(self, data=None):
        if data is None:
            data = ('{0:c}'.format(0))*self.PACK_SIZE
        unpacked_data = (self.PACK_FMT).unpack(data)
        self.revision = unpacked_data[0]
        self.size = unpacked_data[1]
        self.fuse_count = unpacked_data[2]
        self.reserved = ''
        i = 3
        # Pull reserved
        for i in range(i, i + defines.FUSEPROV_RESERVED_MAX_SIZE):
            self.reserved += '{0:c}'.format(unpacked_data[i])
        pass

    def __repr__(self):
        retval = ''
        for attr in sorted(self.__dict__.keys()):
            value = self.__dict__[attr]
            retval += (str(attr) + ': ' + (value if isinstance(value, str) else
                       str(hex(value))) + '\n')
        return (self.__class__.__name__ + ' Properties: ' + '\n' +
                logger.increase_indent(retval))

    def get_packed_data(self):
        values = ([self.revision,
                   self.size,
                   self.fuse_count] +
                   [ord(c) for c in self.reserved])
        return (self.PACK_FMT).pack(*values)


class fuseprov_qfuse_entry(object):

    # Structure object to align and package the fuse entries
    PACK_FMT = struct.Struct('IIIII')

    # Size of the header
    PACK_SIZE = (4 * 5)

    def __init__(self, data=None):
        if data is None:
            data = ('{0:c}'.format(0))*self.PACK_SIZE
        unpacked_data = (self.PACK_FMT).unpack(data)
        self.region_type = unpacked_data[0]
        self.raw_row_address = unpacked_data[1]
        self.lsb_val = unpacked_data[2]
        self.msb_val = unpacked_data[3]
        self.operation = unpacked_data[4]
        pass

    def __repr__(self):
        retval = ''
        for attr in sorted(self.__dict__.keys()):
            value = self.__dict__[attr]
            retval += (str(attr) + ': ' + (value if isinstance(value, str) else
                       str(hex(value))) + '\n')
        return (self.__class__.__name__ + ' Properties: ' + '\n' +
                logger.increase_indent(retval))

    def get_packed_data(self):
        values = [self.region_type,
                  self.raw_row_address,
                  self.lsb_val,
                  self.msb_val,
                  self.operation]
        return (self.PACK_FMT).pack(*values)


class fuseprov_secdat_footer_type(object):

    # Structure object to align and package the footer
    PACK_FMT = struct.Struct('B' * defines.FUSEPROV_HASH_MAX_SIZE)

    # Size of the header
    PACK_SIZE = (1 * defines.FUSEPROV_HASH_MAX_SIZE)

    def __init__(self, data=None):
        if data is None:
            data = ('{0:c}'.format(0))*self.PACK_SIZE
        unpacked_data = (self.PACK_FMT).unpack(data)
        self.hash = ''
        for i in range(0, defines.FUSEPROV_HASH_MAX_SIZE):
            self.hash += '{0:c}'.format(unpacked_data[i])
        pass

    def __repr__(self):
        retval = ''
        for attr in sorted(self.__dict__.keys()):
            value = self.__dict__[attr]
            retval += (str(attr) + ': ' + (value if isinstance(value, str) else
                       str(hex(value))) + '\n')
        return (self.__class__.__name__ + ' Properties: ' + '\n' +
                logger.increase_indent(retval))

    def get_packed_data(self):
        values = [ord(c) for c in self.hash]
        return (self.PACK_FMT).pack(*values)


from sectools.features.fuse_common.parsegen.secdat.secdatfactory import secdatparsegen_mgr
